# test_integration_011_utility.py
import pytest
# requires pytest_dependency to be installed
import unittest
import responses
from fastapi.testclient import TestClient
import requests
import re
import logging
import sys
import os
import base64
import json
from pathlib import Path

# so the directly in which you run pytest or unittest doesn't matter as much
sys.path.append(os.path.abspath('..'))
sys.path.append('tests')

# local
from main import app
from auth import auth
import conf
import testlib

# override dependency injection for authentication to entitlement service
from override_depends import override_require_authorized_user, set_authorize_session, ADMIN_ONLY_SVC, USER_AND_ADMIN_SVC, USER_ONLY_SVC, OTHER_ONLY_SVC

TEST_DATA_DIR = Path(__file__).resolve().parent / 'data'

client = TestClient(app)

@pytest.mark.dependency(name="require_token")
def test_require_token(token):
    assert token is not None, "No token provided on command line"

def test_cloud_provider(cloud_provider):
    """Check CLOUD_PROVIDER """
    assert cloud_provider is not None, "No CLOUD_PROVIDER env set"

@pytest.mark.dependency(name="require_token")
def test_backup(token, data_partition, service_url, cloud_provider):
    """
    Test Backup API
    """
    url = service_url + conf.SERVICE_BASE_PATH + "/backup"
    headers={'Authorization': 'Bearer ' + token, 'data-partition-id': data_partition}
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        pytest.xfail(f"service_url /backup fail for {cloud_provider} {r.status_code} {r.text}")

@pytest.mark.dependency(name="require_token")
def test_bootstrap(token, data_partition, service_url, cloud_provider):
    """
    Test Bootstrap API
    """
    url = service_url + conf.SERVICE_BASE_PATH + "/bootstrap"
    headers={'Authorization': 'Bearer ' + token, 'data-partition-id': data_partition}
    r = requests.post(url, headers=headers)

    if r.status_code == 424 or r.status_code == 405:
        pytest.xfail(f"/bootstrap {cloud_provider} {r.status_code} {r.text}")

    assert r.status_code == 201 or r.status_code == 202, f"/bootstrap {cloud_provider} {r.status_code} {r.text}"
    
@pytest.mark.dependency(name="require_token")
def test_bootstrap_force(token, data_partition, service_url, cloud_provider):
    """
    Test Bootstrap API Force
    """
    url = service_url + conf.SERVICE_BASE_PATH + "/bootstrap"
    params = {'force': True}
    headers={'Authorization': 'Bearer ' + token, 'data-partition-id': data_partition}
    r = requests.post(url, headers=headers, params=params)

    if r.status_code == 424 or r.status_code == 405:
        pytest.xfail(f"/bootstrap {cloud_provider} {r.status_code} {r.text}")

    assert r.status_code == 201 or r.status_code == 202, f"/bootstrap {cloud_provider} {r.status_code} {r.text}"
    