import { Injectable } from '@angular/core'
import { ActivatedRouteSnapshot, CanActivate, Router, RouterStateSnapshot } from '@angular/router'
import { defer, iif, Observable, of } from 'rxjs'
import { catchError, map, mergeMap } from 'rxjs/operators'

@Injectable()
export class AuthGuard implements CanActivate {
  constructor(
    private router: Router,
    private restService: AuthenticatedRestService,
    private clinicianClinicService: ClinicService,
  ) { }

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | Promise<boolean> | boolean {
    const checkAuth$ = defer(() => this.restService.checkAuth())

    return checkAuth$.pipe(
      mergeMap(success =>
        iif(
          () => success,
          this.clinicianClinicService.loadIfNotLoaded().pipe(map(clinic => !!clinic)),
          of(success),
        ),
      ),
      map(success => {
        if (!success) {
          this.router.navigate(['/auth/login'], {
            queryParams: {
              redirectTo: state.url,
            },
          })
        }

        return success
      }),
      catchError(() => {
        this.router.navigate(['/auth/login'], {
          queryParams: {
            redirectTo: state.url,
          },
        })
        return of(null)
      }),
    )
  }
}
